/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2008 Maxime Bizon <mbizon@freebox.fr>
 * Copyright (C) 2008 Florian Fainelli <florian@openwrt.org>
 */

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/gpio_keys.h>
#include <linux/input.h>
#include <linux/pci_ids.h>
#include <linux/platform_device.h>
#include <linux/platform_data/b53.h>
#include <linux/rtl8367.h>
#include <linux/spi/spi.h>
#include <asm/addrspace.h>
#include <bcm63xx_board.h>
#include <bcm63xx_cpu.h>
#include <bcm63xx_regs.h>
#include <bcm63xx_io.h>
#include <bcm63xx_nvram.h>
#include <board_bcm963xx.h>

#include "board_common.h"

#include <uapi/linux/bcm933xx_hcs.h>

#define PFX	"board_bcm963xx: "

#define HCS_OFFSET_128K			0x20000

#define BCM963XX_KEYS_POLL_INTERVAL	20
#define BCM963XX_KEYS_DEBOUNCE_INTERVAL	(BCM963XX_KEYS_POLL_INTERVAL * 3)

#define NB6_GPIO_RTL8367_SDA	18
#define NB6_GPIO_RTL8367_SCK	20

/*
 * known 3368 boards
 */
#ifdef CONFIG_BCM63XX_CPU_3368
static struct board_info __initdata board_cvg834g = {
	.name				= "CVG834G_E15R3921",
	.expected_cpu_id		= 0x3368,

	.has_uart0			= 1,
	.has_uart1			= 1,

	.has_enet0			= 1,
	.has_pci			= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.ephy_reset_gpio		= 36,
	.ephy_reset_gpio_flags		= GPIO_ACTIVE_LOW,
};
#endif /* CONFIG_BCM63XX_CPU_3368 */

/*
 * known 6318 boards
 */
#ifdef CONFIG_BCM63XX_CPU_6318
static struct board_info __initdata board_96318ref = {
	.name				= "96318REF",
	.expected_cpu_id		= 0x6318,

	.has_uart0			= 1,
	.has_pci			= 1,

	.has_usbd			= 1,

	.usbd = {
		.use_fullspeed		= 0,
		.port_no		= 0,
	},

	.has_enetsw			= 1,

	.has_ehci0			= 1,
	.num_usbh_ports			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id = 1,
				.name	= "Port 1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},
};

static struct board_info __initdata board_96318ref_p300 = {
	.name				= "96318REF_P300",
	.expected_cpu_id		= 0x6318,

	.has_uart0			= 1,
	.has_pci			= 1,

	.has_usbd			= 1,

	.usbd = {
		.use_fullspeed		= 0,
		.port_no		= 0,
	},

	.has_enetsw			= 1,

	.has_ehci0			= 1,
	.num_usbh_ports			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id = 1,
				.name	= "Port 1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},
};

static struct sprom_fixup __initdata dsl2751b_e1_fixups[] = {
	{ .offset = 96, .value = 0x2046 },
	{ .offset = 97, .value = 0xfe9d },
	{ .offset = 98, .value = 0x1854 },
	{ .offset = 99, .value = 0xfa59 },
	{ .offset = 112, .value = 0x2046 },
	{ .offset = 113, .value = 0xfe79 },
	{ .offset = 114, .value = 0x17f5 },
	{ .offset = 115, .value = 0xfa47 },
	{ .offset = 161, .value = 0x2222 },
	{ .offset = 162, .value = 0x2222 },
	{ .offset = 169, .value = 0x2222 },
	{ .offset = 170, .value = 0x2222 },
	{ .offset = 171, .value = 0x5555 },
	{ .offset = 172, .value = 0x5555 },
	{ .offset = 173, .value = 0x4444 },
	{ .offset = 174, .value = 0x4444 },
	{ .offset = 175, .value = 0x5555 },
	{ .offset = 176, .value = 0x5555 },
};

static struct board_info __initdata board_dsl_2751b_d1 = {
	.name				= "AW5200B",
	.expected_cpu_id		= 0x6318,

	.has_uart0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.has_enetsw			= 1,

	.has_ohci0			= 1,
	.has_ehci0			= 1,
	.num_usbh_ports			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id = 1,
				.name	= "Port 1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM43217,
		.pci_bus			= 1,
		.pci_dev			= 0,
		.board_fixups			= dsl2751b_e1_fixups,
		.num_board_fixups		= ARRAY_SIZE(dsl2751b_e1_fixups),
	},
};
#endif /* CONFIG_BCM63XX_CPU_6318 */

/*
 * known 6328 boards
 */
#ifdef CONFIG_BCM63XX_CPU_6328
static struct board_info __initdata board_96328avng = {
	.name				= "96328avng",
	.expected_cpu_id		= 0x6328,

	.has_uart0			= 1,
	.has_pci			= 1,
	.has_usbd			= 0,
	.use_fallback_sprom		= 1,

	.usbd = {
		.use_fullspeed		= 0,
		.port_no		= 0,
	},

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id = 1,
				.name	= "Port 1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},
};

static struct sprom_fixup __initdata ar5381u_fixups[] = {
	{ .offset = 97, .value = 0xfee5 },
	{ .offset = 98, .value = 0x157c },
	{ .offset = 99, .value = 0xfae7 },
	{ .offset = 113, .value = 0xfefa },
	{ .offset = 114, .value = 0x15d6 },
	{ .offset = 115, .value = 0xfaf8 },
};

static struct board_info __initdata board_AR5381u = {
	.name					= "96328A-1241N",
	.expected_cpu_id			= 0x6328,

	.has_uart0				= 1,
	.has_pci				= 1,
	.use_fallback_sprom			= 1,
	.has_ohci0				= 1,
	.has_ehci0				= 1,
	.num_usbh_ports				= 1,
	.has_enetsw				= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used		= 1,
				.phy_id		= 1,
				.name		= "Port 1",
			},
			[1] = {
				.used		= 1,
				.phy_id		= 2,
				.name		= "Port 2",
			},
			[2] = {
				.used		= 1,
				.phy_id		= 3,
				.name		= "Port 3",
			},
			[3] = {
				.used		= 1,
				.phy_id		= 4,
				.name		= "Port 4",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM43225,
		.pci_bus			= 1,
		.pci_dev			= 0,
		.board_fixups			= ar5381u_fixups,
		.num_board_fixups		= ARRAY_SIZE(ar5381u_fixups),
	},
};

static struct sprom_fixup __initdata ar5387un_fixups[] = {
	{ .offset = 2, .value = 0x05bb },
	{ .offset = 65, .value = 0x1204 },
	{ .offset = 78, .value = 0x0303 },
	{ .offset = 79, .value = 0x0202 },
	{ .offset = 80, .value = 0xff02 },
	{ .offset = 87, .value = 0x0315 },
	{ .offset = 88, .value = 0x0315 },
	{ .offset = 96, .value = 0x2048 },
	{ .offset = 97, .value = 0xff11 },
	{ .offset = 98, .value = 0x1567 },
	{ .offset = 99, .value = 0xfb24 },
	{ .offset = 100, .value = 0x3e3c },
	{ .offset = 101, .value = 0x4038 },
	{ .offset = 102, .value = 0xfe7f },
	{ .offset = 103, .value = 0x1279 },
	{ .offset = 112, .value = 0x2048 },
	{ .offset = 113, .value = 0xff03 },
	{ .offset = 114, .value = 0x154c },
	{ .offset = 115, .value = 0xfb27 },
	{ .offset = 116, .value = 0x3e3c },
	{ .offset = 117, .value = 0x4038 },
	{ .offset = 118, .value = 0xfe87 },
	{ .offset = 119, .value = 0x1233 },
	{ .offset = 203, .value = 0x2226 },
};

static struct board_info __initdata board_AR5387un = {
	.name					= "96328A-1441N1",
	.expected_cpu_id			= 0x6328,

	.has_uart0				= 1,
	.has_pci				= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0				= 1,
	.has_ehci0				= 1,
	.num_usbh_ports				= 1,
	.has_enetsw				= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used		= 1,
				.phy_id		= 1,
				.name		= "Port 1",
			},
			[1] = {
				.used		= 1,
				.phy_id		= 2,
				.name		= "Port 2",
			},
			[2] = {
				.used		= 1,
				.phy_id		= 3,
				.name		= "Port 3",
			},
			[3] = {
				.used		= 1,
				.phy_id		= 4,
				.name		= "Port 4",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM43225,
		.pci_bus			= 1,
		.pci_dev			= 0,
		.board_fixups			= ar5387un_fixups,
		.num_board_fixups		= ARRAY_SIZE(ar5387un_fixups),
	},
};

static struct board_info __initdata board_963281TAN = {
	.name				= "963281TAN",
	.expected_cpu_id		= 0x6328,

	.has_uart0			= 1,
	.has_pci			= 1,

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id = 1,
				.name	= "Port 1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},
};

static struct board_info __initdata board_A4001N = {
	.name				= "96328dg2x2",
	.expected_cpu_id		= 0x6328,

	.has_uart0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,
	.num_usbh_ports			= 1,
	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id	= 1,
				.name	= "Port 1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM43225,
		.pci_bus			= 1,
		.pci_dev			= 0,
	},
};

static struct board_info __initdata board_A4001N1 = {
	.name					= "963281T_TEF",
	.expected_cpu_id			= 0x6328,

	.has_uart0				= 1,
	.has_pci				= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0				= 1,
	.has_ehci0				= 1,
	.num_usbh_ports				= 1,
	.has_enetsw				= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used		= 1,
				.phy_id		= 1,
				.name		= "Port 1",
			},
			[1] = {
				.used		= 1,
				.phy_id		= 2,
				.name		= "Port 2",
			},
			[2] = {
				.used		= 1,
				.phy_id		= 3,
				.name		= "Port 3",
			},
			[3] = {
				.used		= 1,
				.phy_id		= 4,
				.name		= "Port 4",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM43225,
		.pci_bus			= 1,
		.pci_dev			= 0,
	},
};

static struct board_info __initdata board_dsl_274xb_f1 = {
	.name				= "AW4339U",
	.expected_cpu_id		= 0x6328,

	.has_uart0			= 1,
	.has_pci			= 1,

	.has_caldata			= 1,
	.caldata = {
		{
			.vendor		= PCI_VENDOR_ID_ATHEROS,
			.caldata_offset	= 0x7d1000,
			.slot		= 0,
			.led_pin	= -1,
		},
	},

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id = 1,
				.name	= "Port 4",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 3",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 2",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 1",
			},
		},
	},
};

static struct board_info __initdata board_FAST2704V2 = {
	.name				= "F@ST2704V2",
	.expected_cpu_id		= 0x6328,

	.has_uart0			= 1,
	.has_pci			= 1,
       .use_fallback_sprom             = 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,
	.has_usbd			= 1,

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id = 1,
				.name	= "Port 1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},
        .fallback_sprom = {
                .type                           = SPROM_BCM43225,
                .pci_bus                        = 1,
                .pci_dev                        = 0,
        },
};

static struct board_info __initdata board_FAST2804V7 = {
	.name				= "F@ST2804V7",
	.expected_cpu_id		= 0x6328,

	.has_uart0			= 1,
	.has_pci			= 1,
       .use_fallback_sprom             = 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,
	.has_usbd			= 1,

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id = 1,
				.name	= "Port 1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},
        .fallback_sprom = {
                .type                           = SPROM_BCM43227,
                .pci_bus                        = 1,
                .pci_dev                        = 0,
        },
};
#endif /* CONFIG_BCM63XX_CPU_6328 */

/*
 * known 6338 boards
 */
#ifdef CONFIG_BCM63XX_CPU_6338
static struct board_info __initdata board_96338gw = {
	.name				= "96338GW",
	.expected_cpu_id		= 0x6338,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.enet0 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0			= 1,
};

static struct board_info __initdata board_96338w = {
	.name				= "96338W",
	.expected_cpu_id		= 0x6338,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.enet0 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};

static struct board_info __initdata board_96338w2_e7t = {
	.name				= "96338W2_E7T",
	.expected_cpu_id		= 0x6338,

	.has_enet0			= 1,

	.enet0 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};

static struct board_info __initdata board_rta1320_16m = {
	.name				= "RTA1320_16M",
	.expected_cpu_id		= 0x6338,

	.has_uart0			= 1,
	.has_enet0			= 1,

	.enet0 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};
#endif /* CONFIG_BCM63XX_CPU_6338 */

/*
 * known 6345 boards
 */
#ifdef CONFIG_BCM63XX_CPU_6345
static struct board_info __initdata board_96345gw2 = {
	.name				= "96345GW2",
	.expected_cpu_id		= 0x6345,

	.has_uart0			= 1,
};

static struct board_info __initdata board_rta770bw = {
	.name				= "RTA770BW",
	.expected_cpu_id		= 0x6345,

	.has_uart0			= 1,

	.has_enet0			= 1,

	.enet0 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};

// Actually this board is the very same as the rta770bw,
// where the additional 'b' within the name just
// just indicates 'Annex B'. The ADSL Modem itself is able
// to handle both Annex A as well as Annex B -
// the loaded firmware makes the only difference
static struct board_info __initdata board_rta770w = {
	.name				= "RTA770W",
	.expected_cpu_id		= 0x6345,

	.has_uart0			= 1,

	.has_enet0			= 1,

	.enet0 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};
#endif /* CONFIG_BCM63XX_CPU_6345 */

/*
 * known 6348 boards
 */
#ifdef CONFIG_BCM63XX_CPU_6348
static struct board_info __initdata board_96348r = {
	.name				= "96348R",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
};

static struct board_info __initdata board_96348gw_10 = {
	.name				= "96348GW-10",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0			= 1,
	.has_pccard			= 1,
	.has_ehci0			= 1,

	.has_dsp			= 1,
	.dsp = {
		.gpio_rst		= 6,
		.gpio_int		= 34,
		.cs			= 2,
		.ext_irq		= 2,
	},
};

static struct board_info __initdata board_96348gw_11 = {
	.name				= "96348GW-11",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},


	.has_ohci0 = 1,
	.has_pccard = 1,
	.has_ehci0 = 1,
};

static struct board_info __initdata board_96348gw_10_AR1004G = {
	.name				= "AR1004G",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};


/* BT Voyager 2110 */
static struct board_info __initdata board_V2110 = {
	.name				= "V2110",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};

static struct board_info __initdata board_CPVA502plus = {
	.name				= "CPVA502+",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
	},

	.ephy_reset_gpio		= 4,
	.ephy_reset_gpio_flags		= GPIO_ACTIVE_LOW,
};

static struct board_info __initdata board_ct536_ct5621 = {
	.name				= "CT536_CT5621",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 0,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0 = 1,
	.has_pccard = 1,
	.has_ehci0 = 1,

	.fallback_sprom = {
		.type 				= SPROM_BCM4318,
		.pci_bus			= 0,
		.pci_dev			= 1,
	},
};

static struct board_info __initdata board_96348A_122 = {
	.name				= "96348A-122",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0 = 1,

	.fallback_sprom = {
		.type 				= SPROM_BCM4318,
		.pci_bus			= 0,
		.pci_dev			= 1,
	},
};

static struct board_info __initdata board_96348gw = {
	.name				= "96348GW",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0 = 1,

	.has_dsp			= 1,
	.dsp = {
		.gpio_rst		= 6,
		.gpio_int		= 34,
		.ext_irq		= 2,
		.cs			= 2,
	},
};

static struct board_info __initdata board_gw6200 = {
	.name				= "GW6200",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0 = 1,

	.has_dsp			= 1,
		.dsp = {
		.gpio_rst		= 8, /* FIXME: What is real GPIO here? */
		.gpio_int		= 34,
		.ext_irq		= 2,
		.cs			= 2,
	},
};

static struct board_info __initdata board_gw6000 = {
	.name				= "GW6000",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0 = 1,

	.has_dsp			= 1,
	.dsp = {
		.gpio_rst		= 6,
		.gpio_int		= 34,
		.ext_irq		= 2,
		.cs			= 2,
	},
};



static struct board_info __initdata board_FAST2404 = {
	.name				= "F@ST2404",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0			= 1,
	.has_pccard			= 1,
	.has_ehci0			= 1,
};

static struct board_info __initdata board_FAST2604 = {
	.name						= "F@ST2604",
	.expected_cpu_id			= 0x6348,

	.has_uart0					= 1,
	.has_pci					= 1,
	.has_ohci0					= 1,

	.has_enet1					= 1,
	.enet1 = {
		.has_phy			= 1,
		.phy_id				= 0,
		.force_speed_100		= 1,
		.force_duplex_full		= 1,
	},
};

static struct board_info __initdata board_rta1025w_16 = {
	.name				= "RTA1025W_16",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};


static struct board_info __initdata board_DV201AMR = {
	.name				= "DV201AMR",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0			= 1,

	.has_enet0			= 1,
	.has_enet1			= 1,
	.enet0 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};

static struct board_info __initdata board_96348gw_a = {
	.name				= "96348GW-A",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0 = 1,
};

/* NetGear DG834G v4 */
static struct board_info __initdata board_96348W3 = {
 	.name	 			= "96348W3",
	.expected_cpu_id		= 0x6348,

	.has_uart0	 		= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0	 		= 1,
};

static struct board_info __initdata board_96348_D4PW = {
	.name				= "D-4P-W",
	.expected_cpu_id		= 0x6348,

	.has_enet1			= 1,
	.has_pci			= 1,
	.has_uart0 			= 1,

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};

static struct sprom_fixup __initdata spw500v_fixups[] = {
	{ .offset = 46, .value = 0x3046 },
	{ .offset = 47, .value = 0x15a7 },
	{ .offset = 48, .value = 0xfa89 },
	{ .offset = 49, .value = 0xfe79 },
	{ .offset = 57, .value = 0x6a49 },
};

static struct board_info __initdata board_spw500v = {
	.name				= "SPW500V",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.has_dsp			= 1,
	.dsp = {
		.gpio_rst		= 6,
		.gpio_int		= 34,
		.ext_irq		= 2,
		.cs			= 2,
	},

	.fallback_sprom = {
		.type			= SPROM_BCM4318,
		.pci_bus		= 0,
		.pci_dev		= 1,
		.board_fixups		= spw500v_fixups,
		.num_board_fixups	= ARRAY_SIZE(spw500v_fixups),
	},
};

static struct board_info __initdata board_96348sv = {
	.name				= "MAGIC",
	.expected_cpu_id		= 0x6348,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},
	.enet1 = {
		/* it has BP_ENET_EXTERNAL_PHY */
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0			= 1,
	.has_pccard			= 1,
	.has_ehci0			= 1,

	.has_dsp			= 1,
	.dsp = {
		.gpio_rst		= 25,
		.gpio_int		= 34,
		.cs			= 2,
		.ext_irq		= 2,
	},
};

static struct board_info __initdata board_V2500V_BB = {
	.name				= "V2500V_BB",
        .expected_cpu_id                = 0x6348,

        .has_uart0                      = 1,
        .has_enet0                      = 1,
        .has_enet1                      = 1,
        .has_pci                        = 1,

        .enet0 = {
                .has_phy                = 1,
                .use_internal_phy       = 1,
        },
        .enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
                .force_speed_100        = 1,
                .force_duplex_full      = 1,
	},
};
#endif /* CONFIG_BCM63XX_CPU_6348 */

/*
 * known 6358 boards
 */
#ifdef CONFIG_BCM63XX_CPU_6358
static struct board_info __initdata board_96358vw = {
	.name				= "96358VW",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},


	.has_ohci0 = 1,
	.has_pccard = 1,
	.has_ehci0 = 1,
};

static struct board_info __initdata board_96358vw2 = {
	.name				= "96358VW2",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},


	.has_ohci0 = 1,
	.has_pccard = 1,
	.has_ehci0 = 1,
	.num_usbh_ports			= 2,
};

static struct board_info __initdata board_CPVA642 = {
	.name                           = "CPVA642",
	.expected_cpu_id                = 0x6358,

	.has_uart0			= 1,
	.has_enet1                      = 1,
	.has_pci                        = 1,

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100        = 1,
		.force_duplex_full      = 1,
 	},

	.has_ohci0 = 1,
	.has_ehci0 = 1,
};


static struct board_info __initdata board_AGPFS0 = {
	.name				= "AGPF-S0",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0 = 1,
	.has_ehci0 = 1,
};

static struct board_info __initdata board_DWVS0 = {
	.name				= "DWV-S0",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,

	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.has_ohci0			= 1,
	.has_ehci0			= 1,
};

/* D-Link DSL-274xB revison C2/C3 */
static struct board_info __initdata board_dsl_274xb_rev_c = {
	.name				= "AW4139",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},
};

static struct board_info __initdata board_nb4_ser_r0 = {
	.name				= "NB4-SER-r0",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},


	.has_ohci0 = 1,
	.has_pccard = 1,
	.has_ehci0 = 1,
	.num_usbh_ports			= 2,
};

static struct board_info __initdata board_nb4_fxc_r1 = {
	.name				= "NB4-FXC-r1",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,
	.has_enet0			= 1,
	.has_enet1			= 1,
	.has_pci			= 1,

	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},


	.has_ohci0 = 1,
	.has_pccard = 1,
	.has_ehci0 = 1,
	.num_usbh_ports			= 2,
};

static struct board_info __initdata board_ct6373_1 = {
	.name				= "CT6373-1",
	.expected_cpu_id	= 0x6358,

	.has_uart0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,

	.has_enet1			= 1,
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM4318,
		.pci_bus			= 0,
		.pci_dev			= 1,
	},
};

static struct board_info __initdata board_homehub2a = {
	.name				= "HOMEHUB2A",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,
	.num_usbh_ports			= 2,

	.has_enet1			= 1,
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100        = 1,
		.force_duplex_full      = 1,
	},

	.fallback_sprom = {
		.type 			= SPROM_BCM4322,
		.pci_bus		= 0,
		.pci_dev		= 1,
	},
};

static struct board_info __initdata board_HW520 = {
	.name				= "HW6358GW_B",
	.expected_cpu_id		= 0x6358,

	.has_uart0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,

	.has_enet0			= 1,
	.enet0 = {
		.has_phy		= 1,
		.use_internal_phy	= 1,
	},

	.has_enet1			= 1,
	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM4318,
		.pci_bus			= 0,
		.pci_dev			= 1,
	},
};

static struct board_info __initdata board_HW553 = {
	.name                           = "HW553",
	.expected_cpu_id                = 0x6358,

	.has_uart0			= 1,

	.has_enet1                      = 1,
	.has_pci                        = 1,
	.use_fallback_sprom		= 1,

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100        = 1,
		.force_duplex_full      = 1,
	},

	.has_ohci0 = 1,
	.has_ehci0 = 1,
	.num_usbh_ports			= 2,

	.fallback_sprom = {
		.type 				= SPROM_BCM4318,
		.pci_bus			= 0,
		.pci_dev			= 1,
	},
};

static struct board_info __initdata board_HW556_C = {
	.name					= "HW556_C",
	.expected_cpu_id			= 0x6358,

	.has_uart0				= 1,
	.has_pci				= 1,
	.has_ohci0				= 1,
	.has_ehci0				= 1,
	.num_usbh_ports				= 2,

	.has_caldata				= 1,
	.caldata = {
		{
			.vendor			= PCI_VENDOR_ID_RALINK,
			.caldata_offset		= 0xeffe00,
			.slot			= 1,
			.eeprom			= "rt2x00.eeprom",
		},
	},

	.has_enet1				= 1,
	.enet1 = {
		.has_phy			= 1,
		.phy_id				= 0,
		.force_speed_100		= 1,
		.force_duplex_full		= 1,
	},
};
static struct board_info __initdata board_HW556_A = {
	.name					= "HW556_A",
	.expected_cpu_id			= 0x6358,

	.has_uart0				= 1,
	.has_pci				= 1,
	.has_ohci0				= 1,
	.has_ehci0				= 1,
	.num_usbh_ports				= 2,

	.has_caldata				= 1,
	.caldata = {
		{
			.vendor			= PCI_VENDOR_ID_ATHEROS,
			.caldata_offset		= 0xf7e000,
			.slot			= 1,
			.endian_check		= 1,
			.led_pin		= 2,
		},
	},

	.has_enet1				= 1,
	.enet1 = {
		.has_phy			= 1,
		.phy_id				= 0,
		.force_speed_100		= 1,
		.force_duplex_full		= 1,
	},
};
static struct board_info __initdata board_HW556_B = {
	.name					= "HW556_B",
	.expected_cpu_id			= 0x6358,

	.has_uart0				= 1,
	.has_pci				= 1,
	.has_ohci0				= 1,
	.has_ehci0				= 1,
	.num_usbh_ports				= 2,

	.has_caldata				= 1,
	.caldata = {
		{
			.vendor			= PCI_VENDOR_ID_ATHEROS,
			.caldata_offset		= 0xefe000,
			.slot			= 1,
			.endian_check		= 1,
			.led_pin		= 2,
		},
	},

	.has_enet1				= 1,
	.enet1 = {
		.has_phy			= 1,
		.phy_id				= 0,
		.force_speed_100		= 1,
		.force_duplex_full		= 1,
	},
};

 /* T-Home Speedport W 303V Typ B */
static struct board_info __initdata board_spw303v = {
	.name			= "96358-502V",
	.expected_cpu_id	= 0x6358,

	.has_uart0		= 1,
	.has_enet0		= 1,
	.has_pci		= 1,

	.enet0 = {
		.has_phy 		= 1,
		.use_internal_phy 	= 1,
	},
};

/* D-Link DVA-G3810BN/TL */
static struct board_info __initdata board_DVAG3810BN = {
	.name			= "DVAG3810BN",
	.expected_cpu_id	= 0x6358,

	.has_uart0		= 1,
	.has_enet0		= 1,
	.has_enet1		= 1,
	.has_pci		= 1,

	.enet0 = {
		.has_phy		= 0,
		.use_internal_phy	= 1,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},

	.enet1 = {
		.has_phy		= 1,
		.phy_id			= 0,
		.force_speed_100	= 1,
		.force_duplex_full	= 1,
	},


	.has_ohci0		= 1,
	.has_pccard		= 1,
	.has_ehci0		= 1,
};
#endif /* CONFIG_BCM63XX_CPU_6358 */

#ifdef CONFIG_BCM63XX_CPU_6362
static struct rtl8367_extif_config nb6_rtl8367_extif0_cfg = {
	.mode = RTL8367_EXTIF_MODE_RGMII,
	.txdelay = 1,
	.rxdelay = 5,
	.ability = {
		.force_mode = 1,
		.txpause = 1,
		.rxpause = 1,
		.link = 1,
		.duplex = 1,
		.speed = RTL8367_PORT_SPEED_1000,
	},
};

static struct rtl8367_platform_data nb6_rtl8367_data = {
	.gpio_sda	= NB6_GPIO_RTL8367_SDA,
	.gpio_sck	= NB6_GPIO_RTL8367_SCK,
	.extif0_cfg	= &nb6_rtl8367_extif0_cfg,
};

static struct platform_device nb6_rtl8367_device = {
	.name		= RTL8367_DRIVER_NAME,
	.id		= -1,
	.dev = {
		.platform_data	= &nb6_rtl8367_data,
	}
};

static struct platform_device * __initdata nb6_devices[] = {
	&nb6_rtl8367_device,
};

static struct board_info __initdata board_nb6 = {
	.name				= "NB6",
	.expected_cpu_id		= 0x6362,

	.has_uart0			= 1,

	.has_ohci0			= 1,
	.has_ehci0			= 1,
	.num_usbh_ports			= 2,

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[4] = {
				.used	= 1,
				.phy_id	= 0xff,
				.bypass_link = 1,
				.force_speed = 1000,
				.force_duplex_full = 1,
				.name	= "RGMII",
			},
		},
	},

	.devs = nb6_devices,
	.num_devs = ARRAY_SIZE(nb6_devices),
};

static struct board_info __initdata board_fast2504n = {
	.name				= "F@ST2504n",
	.expected_cpu_id		= 0x6362,

	.has_uart0			= 1,

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id	= 1,
				.name	= "Port 1",
			},

			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "Port 2",
			},

			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "Port 3",
			},

			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "Port 4",
			},
		},
	},
};
#endif /* CONFIG_BCM63XX_CPU_6362 */

/*
 * known 6368 boards
 */
#ifdef CONFIG_BCM63XX_CPU_6368
static struct board_info __initdata board_96368mvwg = {
	.name				= "96368MVWG",
	.expected_cpu_id		= 0x6368,

	.has_uart0			= 1,
	.has_pci			= 1,

	.has_usbd			= 1,

	.usbd = {
		.use_fullspeed		= 0,
		.port_no		= 0,
	},

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "port1",
			},

			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "port2",
			},

			[4] = {
				.used	= 1,
				.phy_id	= 0x12,
				.name	= "port0",
			},

			[5] = {
				.used	= 1,
				.phy_id	= 0x11,
				.name	= "port3",
			},
		},
	},

	.has_ohci0 = 1,
	.has_ehci0 = 1,
};

static struct board_info __initdata board_96368mvngr = {
	.name				= "96368MVNgr",
	.expected_cpu_id		= 0x6368,

	.has_uart0			= 1,
	.has_pci			= 1,
	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id	= 1,
				.name	= "port1",
			},

			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "port2",
			},

			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "port3",
			},

			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "port4",
			},
		},
	},

	.has_ohci0 = 1,
	.has_ehci0 = 1,
};

static struct b53_platform_data DGND3700v1_3800B_b53_pdata = {
	.alias    = "eth0",
};

static struct spi_board_info DGND3700v1_3800B_spi_devices[] = {
	{
		.modalias = "b53-switch",
		.max_speed_hz = 781000,
		.bus_num = 0,
		.chip_select = 1,
		.platform_data = &DGND3700v1_3800B_b53_pdata,
	}
};

static struct board_info __initdata board_DGND3700v1_3800B = {
	.name				= "DGND3700v1_3800B",
	.expected_cpu_id		= 0x6368,

	.has_uart0			= 1,
	.has_pci			= 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,
	.num_usbh_ports			= 2,

	.has_enetsw			= 1,
	.enetsw = {
		.used_ports = {
			[5] = {
				.used  = 1,
				.phy_id  = 0xff,
				.bypass_link = 1,
				.force_speed = 1000,
				.force_duplex_full = 1,
				.name  = "RGMII",
			},
		},
	},

	.spis = DGND3700v1_3800B_spi_devices,
	.num_spis = ARRAY_SIZE(DGND3700v1_3800B_spi_devices),
};

static struct board_info __initdata board_HG655b = {
	.name				= "HW65x",
	.expected_cpu_id		= 0x6368,

	.has_uart0			= 1,
	.has_pci			= 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,
	.num_usbh_ports			= 2,

	.has_caldata			= 1,
	.caldata = {
		{
			.vendor			= PCI_VENDOR_ID_RALINK,
			.caldata_offset		= 0x7c0000,
			.slot			= 1,
			.eeprom			= "rt2x00.eeprom",
		},
	},

	.has_enetsw			= 1,
	.enetsw = {
		.used_ports = {
			[0] = {
				.used		= 1,
				.phy_id		= 1,
				.name		= "port1",
			},
			[1] = {
				.used		= 1,
				.phy_id		= 2,
				.name		= "port2",
			},
			[2] = {
				.used		= 1,
				.phy_id		= 3,
				.name		= "port3",
			},
			[3] = {
				.used		= 1,
				.phy_id		= 4,
				.name		= "port4",
			},
		},
	},
};

static struct sprom_fixup __initdata vr3025u_fixups[] = {
	{ .offset = 97, .value = 0xfeb3 },
	{ .offset = 98, .value = 0x1618 },
	{ .offset = 99, .value = 0xfab0 },
	{ .offset = 113, .value = 0xfed1 },
	{ .offset = 114, .value = 0x1609 },
	{ .offset = 115, .value = 0xfad9 },
};

static struct board_info __initdata board_P870HW51A_V2 = {
	.name				= "P870HW-51a_v2",
	.expected_cpu_id		= 0x6368,

	.has_uart0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,

	.has_enetsw			= 1,
	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id	= 1,
				.name	= "port1",
			},
			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "port2",
			},
			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "port3",
			},
			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "port4",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM4318,
		.pci_bus			= 0,
		.pci_dev			= 1,
	},
};

static struct board_info __initdata board_VR3025u = {
	.name					= "96368M-1541N",
	.expected_cpu_id		= 0x6368,

	.has_uart0				= 1,
	.has_pci				= 1,
	.use_fallback_sprom			= 1,
	.has_ohci0				= 1,
	.has_ehci0				= 1,

	.has_enetsw				= 1,
	.enetsw = {
		.used_ports = {
			[0] = {
				.used		= 1,
				.phy_id		= 1,
				.name		= "port1",
			},
			[1] = {
				.used		= 1,
				.phy_id		= 2,
				.name		= "port2",
			},
			[2] = {
				.used		= 1,
				.phy_id		= 3,
				.name		= "port3",
			},
			[3] = {
				.used		= 1,
				.phy_id		= 4,
				.name		= "port4",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM43222,
		.pci_bus			= 0,
		.pci_dev			= 1,
		.board_fixups			= vr3025u_fixups,
		.num_board_fixups		= ARRAY_SIZE(vr3025u_fixups),
	},
};

static struct sprom_fixup __initdata vr3025un_fixups[] = {
	{ .offset = 97, .value = 0xfeb3 },
	{ .offset = 98, .value = 0x1618 },
	{ .offset = 99, .value = 0xfab0 },
	{ .offset = 113, .value = 0xfed1 },
	{ .offset = 114, .value = 0x1609 },
	{ .offset = 115, .value = 0xfad9 },
};

static struct board_info __initdata board_VR3025un = {
	.name					= "96368M-1341N",
	.expected_cpu_id		= 0x6368,

	.has_uart0				= 1,
	.has_pci				= 1,
	.use_fallback_sprom			= 1,
	.has_ohci0				= 1,
	.has_ehci0				= 1,

	.has_enetsw				= 1,
	.enetsw = {
		.used_ports = {
			[0] = {
				.used		= 1,
				.phy_id		= 1,
				.name		= "port1",
			},
			[1] = {
				.used		= 1,
				.phy_id		= 2,
				.name		= "port2",
			},
			[2] = {
				.used		= 1,
				.phy_id		= 3,
				.name		= "port3",
			},
			[3] = {
				.used		= 1,
				.phy_id		= 4,
				.name		= "port4",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM43222,
		.pci_bus			= 0,
		.pci_dev			= 1,
		.board_fixups			= vr3025un_fixups,
		.num_board_fixups		= ARRAY_SIZE(vr3025un_fixups),
	},
};

static struct b53_platform_data WAP5813n_b53_pdata = {
	.alias = "eth0",
};

static struct spi_board_info WAP5813n_spi_devices[] = {
	{
		.modalias = "b53-switch",
		.max_speed_hz = 781000,
		.bus_num = 0,
		.chip_select = 0,
		.platform_data = &WAP5813n_b53_pdata,
	}
};

static struct sprom_fixup __initdata wap5813n_fixups[] = {
	{ .offset = 97, .value = 0xfeed },
	{ .offset = 98, .value = 0x15d1 },
	{ .offset = 99, .value = 0xfb0d },
	{ .offset = 113, .value = 0xfef7 },
	{ .offset = 114, .value = 0x15f7 },
	{ .offset = 115, .value = 0xfb1a },
};

static struct board_info __initdata board_WAP5813n = {
	.name				= "96369R-1231N",
	.expected_cpu_id		= 0x6368,

	.has_uart0			= 1,
	.has_pci			= 1,
	.use_fallback_sprom		= 1,
	.has_ohci0			= 1,
	.has_ehci0			= 1,

	.has_enetsw			= 1,
	.enetsw = {
		.used_ports = {
			[4] = {
				.used = 1,
				.phy_id = 0xff,
				.bypass_link = 1,
				.force_speed = 1000,
				.force_duplex_full = 1,
				.name = "RGMII",
			},
		},
	},

	.fallback_sprom = {
		.type 				= SPROM_BCM43222,
		.pci_bus			= 0,
		.pci_dev			= 1,
		.board_fixups			= wap5813n_fixups,
		.num_board_fixups		= ARRAY_SIZE(wap5813n_fixups),
	},

	.spis = WAP5813n_spi_devices,
	.num_spis = ARRAY_SIZE(WAP5813n_spi_devices),
};
#endif /* CONFIG_BCM63XX_CPU_6368 */

/*
 * known 63268/63269 boards
 */
#ifdef CONFIG_BCM63XX_CPU_63268
static struct board_info __initdata board_963268bu_p300 = {
	.name				= "963268BU_P300",
	.expected_cpu_id		= 0x63268,

	.has_uart0			= 1,

	.has_ehci0			= 1,
	.has_ohci0			= 1,
	.num_usbh_ports			= 1,

	.has_usbd			= 1,

	.usbd = {
		.use_fullspeed		= 0,
		.port_no		= 0,
	},

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id	= 17,
				.name	= "FE1",
			},

			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "GbE2",
			},

			[4] = {
				.used	= 1,
				.phy_id	= 0,
				.name	= "GbE3",
			},

			[5] = {
				.used	= 1,
				.phy_id	= 1,
				.name	= "GbE1",
			},

			[6] = {
				.used	= 1,
				.phy_id	= 24,
				.name	= "GbE4",
			},

			[7] = {
				.used	= 1,
				.phy_id	= 25,
				.name	= "GbE5",
			},
		},
	},
};

static struct board_info __initdata board_963269bhr = {
	.name				= "963269BHR",
	.expected_cpu_id		= 0x63268,

	.has_uart0			= 1,

	.has_pci			= 1,

	.has_ehci0			= 1,

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id	= 1,
				.name	= "port1",
			},

			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "port2",
			},

			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "port3",
			},

			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "port4",
			},
		},
	},
};

static struct board_info __initdata board_vw6339gu = {
	.name				= "VW6339GU",
	.expected_cpu_id		= 0x63268,

	.has_uart0			= 1,

	.has_ehci0			= 1,
	.has_ohci0			= 1,
	.num_usbh_ports			= 1,

	.has_enetsw			= 1,

	.enetsw = {
		.used_ports = {
			[0] = {
				.used	= 1,
				.phy_id	= 1,
				.name	= "LAN2",
			},

			[1] = {
				.used	= 1,
				.phy_id	= 2,
				.name	= "LAN3",
			},

			[2] = {
				.used	= 1,
				.phy_id	= 3,
				.name	= "LAN4",
			},

			[3] = {
				.used	= 1,
				.phy_id	= 4,
				.name	= "LAN1",
			},

			[4] = {
				.used	= 1,
				.phy_id	= 7,
				.name	= "WAN",
			},
		},
	},
};
#endif /* CONFIG_BCM63XX_CPU_63268 */

/*
 * all boards
 */
static const struct board_info __initconst *bcm963xx_boards[] = {
#ifdef CONFIG_BCM63XX_CPU_3368
	&board_cvg834g,
#endif
#ifdef CONFIG_BCM63XX_CPU_6318
	&board_96318ref,
	&board_96318ref_p300,
	&board_dsl_2751b_d1,
#endif
#ifdef CONFIG_BCM63XX_CPU_6328
	&board_96328avng,
	&board_AR5381u,
	&board_AR5387un,
	&board_963281TAN,
	&board_A4001N,
	&board_A4001N1,
	&board_dsl_274xb_f1,
	&board_FAST2704V2,
	&board_FAST2804V7,
#endif
#ifdef CONFIG_BCM63XX_CPU_6338
	&board_96338gw,
	&board_96338w,
	&board_96338w2_e7t,
	&board_rta1320_16m,
#endif
#ifdef CONFIG_BCM63XX_CPU_6345
	&board_96345gw2,
	&board_rta770bw,
	&board_rta770w,
#endif
#ifdef CONFIG_BCM63XX_CPU_6348
	&board_96348r,
	&board_96348gw,
	&board_gw6000,
	&board_gw6200,
	&board_96348gw_10,
	&board_96348gw_11,
	&board_FAST2404,
	&board_FAST2604,
	&board_DV201AMR,
	&board_96348gw_a,
	&board_rta1025w_16,
	&board_96348_D4PW,
	&board_spw500v,
	&board_96348sv,
	&board_V2500V_BB,
	&board_V2110,
	&board_ct536_ct5621,
	&board_96348A_122,
	&board_CPVA502plus,
	&board_96348W3,
	&board_96348gw_10_AR1004G,
#endif

#ifdef CONFIG_BCM63XX_CPU_6358
	&board_96358vw,
	&board_96358vw2,
	&board_AGPFS0,
	&board_CPVA642,
	&board_DWVS0,
	&board_dsl_274xb_rev_c,
	&board_nb4_ser_r0,
	&board_nb4_fxc_r1,
	&board_ct6373_1,
	&board_homehub2a,
	&board_HW520,
	&board_HW553,
	&board_HW556_A,
	&board_HW556_B,
	&board_HW556_C,
	&board_spw303v,
	&board_DVAG3810BN,
#endif

#ifdef CONFIG_BCM63XX_CPU_6362
	&board_nb6,
	&board_fast2504n,
#endif

#ifdef CONFIG_BCM63XX_CPU_6368
	&board_96368mvwg,
	&board_96368mvngr,
	&board_DGND3700v1_3800B,
	&board_HG655b,
	&board_P870HW51A_V2,
	&board_VR3025u,
	&board_VR3025un,
	&board_WAP5813n,
#endif
#ifdef CONFIG_BCM63XX_CPU_63268
	&board_963268bu_p300,
	&board_963269bhr,
	&board_vw6339gu,
#endif
};

static struct of_device_id const bcm963xx_boards_dt[] = {
#ifdef CONFIG_OF
#ifdef CONFIG_BCM63XX_CPU_3368
	{ .compatible = "netgear,cvg834g", .data = &board_cvg834g, },
#endif
#ifdef CONFIG_BCM63XX_CPU_6318
	{ .compatible = "brcm,bcm96318ref", .data = &board_96318ref, },
	{ .compatible = "brcm,bcm96318ref_p300", .data = &board_96318ref_p300, },
	{ .compatible = "d-link,dsl-275xb-d", .data = &board_dsl_2751b_d1, },
#endif
#ifdef CONFIG_BCM63XX_CPU_6328
	{ .compatible = "adb,a4001n", .data = &board_A4001N, },
	{ .compatible = "adb,a4001n1", .data = &board_A4001N1, },
	{ .compatible = "brcm,bcm963281TAN", .data = &board_963281TAN, },
	{ .compatible = "brcm,bcm96328avng", .data = &board_96328avng, },
	{ .compatible = "comtrend,ar-5381u", .data = &board_AR5381u, },
	{ .compatible = "comtrend,ar-5387un", .data = &board_AR5387un, },
	{ .compatible = "d-link,dsl-274xb-f", .data = &board_dsl_274xb_f1, },
	{ .compatible = "sagem,f@st2704v2", .data = &board_FAST2704V2, },
	{ .compatible = "sagem,f@st2804v7", .data = &board_FAST2804V7, },
#endif
#ifdef CONFIG_BCM63XX_CPU_6338
	{ .compatible = "brcm,bcm96338gw", .data = &board_96338gw, },
	{ .compatible = "brcm,bcm96338w", .data = &board_96338w, },
	{ .compatible = "dynalink,rta1320", .data = &board_rta1320_16m, },
	{ .compatible = "d-link,dsl-2640u", .data = &board_96338w2_e7t, },
#endif
#ifdef CONFIG_BCM63XX_CPU_6345
	{ .compatible = "brcm,bcm96345gw2", .data = &board_96345gw2, },
	{ .compatible = "dynalink,rta770bw", .data = &board_rta770bw, },
	{ .compatible = "dynalink,rta770w", .data = &board_rta770w, },
#endif
#ifdef CONFIG_BCM63XX_CPU_6348
	{ .compatible = "asmax,ar1004g", .data = &board_96348gw_10_AR1004G, },
	{ .compatible = "belkin,f5d7633", .data = &board_96348gw_10, },
	{ .compatible = "brcm,bcm96348r", .data = &board_96348r, },
	{ .compatible = "brcm,bcm96348gw-10", .data = &board_96348gw_10, },
	{ .compatible = "brcm,bcm96348gw-11", .data = &board_96348gw_11, },
	{ .compatible = "brcm,bcm96348gw-a", .data = &board_96348gw_a, },
	{ .compatible = "bt,v2110", .data = &board_V2110, },
	{ .compatible = "bt,v2500v-bb", .data = &board_V2500V_BB, },
	{ .compatible = "comtrend,ct-536+", .data = &board_ct536_ct5621, },
	{ .compatible = "comtrend,ct-5365", .data = &board_96348A_122, },
	{ .compatible = "comtrend,ct-5621", .data = &board_ct536_ct5621, },
	{ .compatible = "d-link,dsl-2640b-b", .data = &board_96348_D4PW, },
	{ .compatible = "davolink,dv-201amr", .data = &board_DV201AMR, },
	{ .compatible = "dynalink,rta1025w", .data = &board_rta1025w_16, },
	{ .compatible = "netgear,dg834gtpn", .data = &board_96348gw_10, },
 	{ .compatible = "netgear,dg834g-v4", .data = &board_96348W3, },
	{ .compatible = "sagem,f@st2404", .data = &board_FAST2404, },
 	{ .compatible = "sagem,f@st2604", .data = &board_FAST2604, },
	{ .compatible = "t-com,spw500v", .data = &board_spw500v, },
	{ .compatible = "tecom,gw6000", .data = &board_gw6000, },
	{ .compatible = "tecom,gw6200", .data = &board_gw6200, },
	{ .compatible = "telsey,cpva502+", .data = &board_CPVA502plus, },
	{ .compatible = "telsey,magic", .data = &board_96348sv, },
	{ .compatible = "tp-link,td-w8900gb", .data = &board_96348gw_11, },
	{ .compatible = "usr,9108", .data = &board_96348gw_a, },
#endif
#ifdef CONFIG_BCM63XX_CPU_6358
	{ .compatible = "alcatel,rg100a", .data = &board_96358vw2, },
	{ .compatible = "brcm,bcm96358vw", .data = &board_96358vw, },
	{ .compatible = "brcm,bcm96358vw2", .data = &board_96358vw2, },
	{ .compatible = "comtrend,ct-6373", .data = &board_ct6373_1, },
	{ .compatible = "d-link,dsl-274xb-c2", .data = &board_dsl_274xb_rev_c, },
	{ .compatible = "d-link,dsl-2650u", .data = &board_96358vw2, },
	{ .compatible = "d-link,dva-g3810bn/tl", .data = &board_DVAG3810BN, },
	{ .compatible = "huawei,hg520v", .data = &board_HW520, },
	{ .compatible = "huawei,hg553", .data = &board_HW553, },
	{ .compatible = "huawei,hg556a-a", .data = &board_HW556_A, },
	{ .compatible = "huawei,hg556a-b", .data = &board_HW556_B, },
	{ .compatible = "huawei,hg556a-c", .data = &board_HW556_C, },
	{ .compatible = "pirelli,a226g", .data = &board_DWVS0, },
	{ .compatible = "pirelli,a226m", .data = &board_DWVS0, },
	{ .compatible = "pirelli,a226m-fwb", .data = &board_DWVS0, },
	{ .compatible = "pirelli,agpf-s0", .data = &board_AGPFS0, },
	{ .compatible = "sfr,nb4-ser-r0", .data = &board_nb4_ser_r0, },
	{ .compatible = "sfr,nb4-fxc-r1", .data = &board_nb4_fxc_r1, },
	{ .compatible = "t-com,spw303v", .data = &board_spw303v, },
	{ .compatible = "telsey,cpva642", .data = &board_CPVA642, },
	{ .compatible = "thomson,homehub2a", .data = &board_homehub2a, },
#endif
#ifdef CONFIG_BCM63XX_CPU_6362
	{ .compatible = "sagem,f@st2504n", .data = &board_fast2504n, },
	{ .compatible = "sfr,nb6-ser-r0", .data = &board_nb6, },
#endif
#ifdef CONFIG_BCM63XX_CPU_6368
	{ .compatible = "brcm,bcm96368mvngr", .data = &board_96368mvngr, },
	{ .compatible = "brcm,bcm96368mvwg", .data = &board_96368mvwg, },
	{ .compatible = "comtrend,vr-3025u", .data = &board_VR3025u, },
	{ .compatible = "comtrend,vr-3025un", .data = &board_VR3025un, },
	{ .compatible = "comtrend,wap-5813n", .data = &board_WAP5813n, },
	{ .compatible = "huawei,hg655b", .data = &board_HG655b, },
	{ .compatible = "netgear,dgnd3700v1", .data = &board_DGND3700v1_3800B, },
	{ .compatible = "zyxel,p870hw-51a-v2", .data = &board_P870HW51A_V2, },
#endif
#ifdef CONFIG_BCM63XX_CPU_63268
	{ .compatible = "brcm,bcm963268bu_p300", .data = &board_963268bu_p300, },
	{ .compatible = "brcm,bcm963269bhr", .data = &board_963269bhr, },
	{ .compatible = "inteno,vg50", .data = &board_vw6339gu, },
#endif
#endif /* CONFIG_OF */
	{ },
};

/*
 * early init callback, read nvram data from flash and checksum it
 */
void __init board_bcm963xx_init(void)
{
	unsigned int i;
	u8 *boot_addr, *cfe;
	char cfe_version[32];
	char *board_name = NULL;
	u32 val;
	struct bcm_hcs *hcs;
	const struct of_device_id *board_match;

	/* read base address of boot chip select (0)
	 * 6328/6362 do not have MPI but boot from a fixed address
	 */
	if (BCMCPU_IS_6318() || BCMCPU_IS_6328() || BCMCPU_IS_6362() || BCMCPU_IS_63268()) {
		val = 0x18000000;
	} else {
		val = bcm_mpi_readl(MPI_CSBASE_REG(0));
		val &= MPI_CSBASE_BASE_MASK;
	}
	boot_addr = (u8 *)KSEG1ADDR(val);
	printk(KERN_INFO PFX "Boot address 0x%08x\n",(unsigned int)boot_addr);

	/* BT Voyager 2500V (RTA1046VW PCB) has 8 Meg flash used as two */
	/* banks of 4 Meg. The byte at 0xBF800000 identifies the back to use.*/
	/* Loading firmware from the CFE Prompt always loads to Bank 0 */
	/* Do an early check of CFE and then select bank 0 */

	if (boot_addr == (u8 *)0xbf800000) {
		u8 *tmp_boot_addr = (u8*)0xbfc00000;

		bcm63xx_nvram_init(tmp_boot_addr + BCM963XX_NVRAM_OFFSET);
		if (!strcmp(bcm63xx_nvram_get_name(), "V2500V_BB")) {
			printk(KERN_INFO PFX "V2500V: nvram bank 0\n");
			boot_addr = tmp_boot_addr;
		}
	}

	/* dump cfe version */
	cfe = boot_addr + BCM963XX_CFE_VERSION_OFFSET;
	if (strstarts(cfe, "cfe-")) {
		if(cfe[4] == 'v') {
			if(cfe[5] == 'd')
				snprintf(cfe_version, 11, "%s", (char *) &cfe[5]);
			else if (cfe[10] > 0)
				snprintf(cfe_version, sizeof(cfe_version), "%u.%u.%u-%u.%u-%u",
					 cfe[5], cfe[6], cfe[7], cfe[8], cfe[9], cfe[10]);
			else
				snprintf(cfe_version, sizeof(cfe_version), "%u.%u.%u-%u.%u",
					 cfe[5], cfe[6], cfe[7], cfe[8], cfe[9]);
		} else {
			snprintf(cfe_version, 12, "%s", (char *) &cfe[4]);
		}
	} else
		strcpy(cfe_version, "unknown");
	printk(KERN_INFO PFX "CFE version: %s\n", cfe_version);

	bcm63xx_nvram_init(boot_addr + BCM963XX_NVRAM_OFFSET);

	if (BCMCPU_IS_3368()) {
		hcs = (struct bcm_hcs *)boot_addr;
		board_name = hcs->filename;
	} else {
		board_name = bcm63xx_nvram_get_name();
	}

printk(KERN_INFO PFX "0:%s\n",bcm63xx_nvram_get_name());
printk(KERN_INFO PFX "1:%s\n",board_name);
	/* find board by name */
	for (i = 0; i < ARRAY_SIZE(bcm963xx_boards); i++) {
		if (strncmp(board_name, bcm963xx_boards[i]->name, 16))
			continue;
		/* copy, board desc array is marked initdata */
		board_early_setup(bcm963xx_boards[i],
				  bcm63xx_nvram_get_mac_address);
//		break;
		return;
	}
printk(KERN_INFO PFX "2:\n");
	/* find board by compat */
	board_match = bcm63xx_match_board(bcm963xx_boards_dt);
	/* find board by name */
	if (board_match) {
		board_early_setup(board_match->data,
				  bcm63xx_nvram_get_mac_address);
		return;
	}
printk(KERN_INFO PFX "3:\n");

	/* warn if board is not found, will complain later */
	if (i == ARRAY_SIZE(bcm963xx_boards)) {
		char name[17];
		memcpy(name, board_name, 16);
		name[16] = 0;
		printk(KERN_ERR PFX "unknown bcm963xx board: %s\n",
		       name);
	}
}
